//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x000000,//背景色
  },
}
//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Title: "assets/title-scene.png",
  Game: "assets/game-scene.png",
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  
  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new TitleScene());
  }
}

//タイトルシーン
class TitleScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Title].texture;
    this.addChild(img)

    const style = new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 40,
      fill: 0xffffff,
      fontWeight: 'bold',
      stroke: 0xFF0000,//縁取りの色
      strokeThickness: 4,//縁取りの太さ
    });
    const p = new PIXI.Text("おばけ", style);
    p.anchor.set(0.5, 0.5);
    p.position.set(Config.Screen.Width * 0.5, Config.Screen.Height * 0.15);
    this.addChild(p);

    this.isFading = true; 

    //フェードイン
    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    //タッチイベント
    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 
      
      //フェードアウト
      new FadeOut(this, 0x000000, 40, () => {
        core.replaceScene(new MainScene());//コールバックでシーン切り替え
      });      
    });
  }
  update(delta){
    super.update(delta);   
  }
}


//メインシーン
class MainScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Game].texture;
    this.addChild(img)

    this.isFading = true; 

    //フェードイン
    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    //タッチイベント
    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 

      //フェードアウト
      new FadeOut(this, 0x000000, 40, () => {
        core.replaceScene(new TitleScene());//コールバックでシーン切り替え
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}

//フェードイン
class FadeIn extends Graphics {
  constructor(container, color, frameLength, callback){
    super();
    container.addChild(this);
    this.rectFill(0, 0, Config.Screen.Width, Config.Screen.Height, color);
    this.step = 1 / frameLength;
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  update(delta){
    super.update(delta);
    this.alpha -= this.step;
    if(this.alpha <= 0){
      this.onFinished();
      this.destroy();
      return;
    }   
  }
}

//フェードアウト
class FadeOut extends Graphics {
  constructor(container, color, frameLength, callback){
    super();
    container.addChild(this);
    this.rectFill(0, 0, Config.Screen.Width, Config.Screen.Height, color);
    this.step = 1 / frameLength;
    this.alpha = 0;
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  update(delta){
    super.update(delta);
    this.alpha += this.step;
    if(this.alpha >= 1){
      this.onFinished();
      this.destroy();
      return;
    }   
  }
}